void PhysicsSetTransformSpin (const NewtonBody* body, const dFloat* matrix, int threadIndex)
{
	// get the graphic entity form the rigid body
	ENTITY *entity = (ENTITY*)NewtonBodyGetUserData(body);

	// set the transformation matrix for this rigid body
	ent_setmatrix_rb(entity, matrix);
}

action ball_16_newt()
{
	wait(3);
	// 0: (prepare) vars and get radius of graphical entity
	float inertia[3];
	float centerofmass[3];
	
	VECTOR vmin; vec_for_min(&vmin, my);
	VECTOR vmax; vec_for_max(&vmax, my);
	float radiusx = (vmax.x - vmin.x) / 2.0;
	float radiusy = (vmax.y - vmin.y) / 2.0;
	float radiusz = (vmax.z - vmin.z) / 2.0;
	
	// 1: create collision
	NewtonCollision* collision = NewtonCreateSphere (nworld, radiusx * QUANTTOMETER,	radiusy * QUANTTOMETER,	radiusz * QUANTTOMETER, NULL);
	
	// 2: add collision to body  
	NewtonBody* rigidBody = NewtonCreateBody (nworld, collision); 
	
	// 3: save the pointer to the graphic entity
	my.skill99 = rigidBody;
	NewtonBodySetUserData(rigidBody, my);
	
	// 4: add standard wrapper's destructor: it deletes graphical entity when you remove the body 
	NewtonBodySetDestructorCallback(rigidBody, onsetdestructor);
	
	// 5: get inertia and center of collition
	NewtonConvexCollisionCalculateInertialMatrix(collision, inertia, centerofmass); 
	
	// 6: set mass matrix, using mass(20) and inertia in 3 directions
	NewtonBodySetMassMatrix(rigidBody, 20, 20 * inertia[0], 20 * inertia[1], 20 * inertia[2]);
	
	// 7: release collision, as we dont need it anymore
	NewtonReleaseCollision(nworld, collision);
	
	// 8: set the transform callback function
	NewtonBodySetTransformCallback (rigidBody, PhysicsSetTransformSpin);
	
	// 9: get transformation matrix of the graphical entity...
	float m[16]; ent_getmatrix_rb(my, m);
	// 10: ... and set it to body
	NewtonBodySetMatrix(rigidBody, m);
	
	// 11: set bodie's angular damping to 0
	NewtonBodySetAngularDamping(rigidBody, vectorf(0,0,0));
	
	// 12: add omega force to rotate ball
	NewtonBodySetOmega (rigidBody, vectorf(0,0,-30));
}