// lite-c newton demo (c) 2007 thomas oppl
// updated for Newton2 by VeTaL

//#######################################################################################
//####################################################################################### DEBUG UTILITIES
//#######################################################################################

//--------------------------------------------------------------------------------------- drawaxes()
// draw the local coordinate axes of an entity
//---------------------------------------------------------------------------------------
void drawaxes(ENTITY* entity)
{
	VECTOR xaxis; vec_set(&xaxis, vector(50, 0, 0));
	VECTOR yaxis; vec_set(&yaxis, vector(0, 50, 0));
	VECTOR zaxis; vec_set(&zaxis, vector(0, 0, 50));
	
	vec_rotate(&xaxis, &entity->pan);
	vec_rotate(&yaxis, &entity->pan);
	vec_rotate(&zaxis, &entity->pan);
	vec_add(&xaxis, &entity->x);
	vec_add(&yaxis, &entity->x);
	vec_add(&zaxis, &entity->x);
	
	draw_line3d(&entity->x, NULL, 100);
	draw_line3d(&entity->x, vector(0,0,255), 100); 
	draw_line3d(&xaxis, vector(0,0,255), 100);
	draw_line3d(&entity->x, NULL, 100);
	draw_line3d(&entity->x, vector(0,255,0), 100); 
	draw_line3d(&yaxis, vector(0,255,0), 100); 
	draw_line3d(&entity->x, NULL, 100);
	draw_line3d(&entity->x, vector(255,0,0), 100); 
	draw_line3d(&zaxis, vector(255,0,0), 100); 
}

// draw debug contact
//
#define MAX_DEBUG_CONTACTS	(1024 * 16)
int debugPointsCount;
var showContacts;

void DebugDrawContact (VECTOR* point_vec, VECTOR* normal_vec, float lenght)
{
	if (debugPointsCount < MAX_DEBUG_CONTACTS) {
		VECTOR* temp_vec;
		vec_scale(normal_vec,lenght);
		vec_set(temp_vec,point_vec);
		vec_add(temp_vec,normal_vec);
		draw_point3d(temp_vec,vector(225,0,0),80,5);
		
		vec_set(temp_vec,point_vec);
		vec_sub(temp_vec,normal_vec);
		draw_point3d(temp_vec,vector(225,0,0),80,5);
		
		debugPointsCount ++;
	}
}

void ShowContacts (const NewtonJoint* contactJoint)
{
	// the application can implement some kind of contact debug here
	if (showContacts) {
		
		nworld = NewtonBodyGetWorld(NewtonJointGetBody0(contactJoint));
		NewtonWorldCriticalSectionLock(nworld);
		void* contact;
		for (contact = NewtonContactJointGetFirstContact (contactJoint); contact; contact = NewtonContactJointGetNextContact (contactJoint, contact)) {
			VECTOR* point_vec;
			VECTOR* normal_vec;	
			
			NewtonMaterial* nmaterial;
			
			nmaterial = NewtonContactGetMaterial (contact);
			NewtonMaterialGetContactPositionAndNormal (nmaterial, point_vec, normal_vec);
			
			// if we are display debug info we need to block other threads from writing the data at the same time
			DebugDrawContact (point_vec, normal_vec, 5);
		}
		NewtonWorldCriticalSectionUnlock(nworld);
	}
}

//--------------------------------------------------------------------------------------- foreachpolygon()
// gets called for each polygon and draws it
//---------------------------------------------------------------------------------------
void foreachpolygon(NewtonBody *body, int vertexcount, float *facearray, int faceid)
{
	int i;
	for(i = 0; i < vertexcount; i++)
	{
		VECTOR v;
		v.x = facearray[i*3+0] * METERTOQUANT;
		v.y = facearray[i*3+1] * METERTOQUANT;
		v.z = facearray[i*3+2] * METERTOQUANT;
		
		if(i == 0)
		{
			draw_line3d(&v, NULL, 100);
			draw_line3d(&v, vector(0,255,255), 100);
		}
		else
		{
			draw_line3d(&v, vector(0,255,255), 100); 
		}
	}
}

//--------------------------------------------------------------------------------------- newton_debugdisplay()
// display collision debug geometry
//---------------------------------------------------------------------------------------
void newton_debugdisplay()
{
	NewtonBody* body;
	while (newtondebugdisplaymode)
	{
		if ((newton_running) && (nworld!=0))
		{			
			for (body = NewtonWorldGetFirstBody(nworld); body; body = NewtonWorldGetNextBody(nworld, body))
			{
				float mass, ixx, iyy, izz;
				NewtonBodyGetMassMatrix(body, &mass, &ixx, &iyy, &izz);
				if(mass != 0) // not static
				{
					if (!NewtonBodyGetSleepState(body)) // non-sleeping
					{
						NewtonCollision* collision = NewtonBodyGetCollision(body);
						
						D3DXMATRIX body_matrix;
						NewtonBodyGetMatrix(body, body_matrix);
						
						NewtonCollisionForEachPolygonDo(collision, body_matrix, foreachpolygon, NULL);
						
						ENTITY* entity = (ENTITY*)NewtonBodyGetUserData(body);			
						if (entity) drawaxes(entity);
					}
				}
			}
		}	
		else
		{
			break;
		}	
		wait(1);
	}
}


//--------------------------------------------------------------------------------------- newton_toggledebugdisplay()
// toggle the debug display
//---------------------------------------------------------------------------------------
void newton_toggledebugdisplay()
{
	newtondebugdisplaymode = 1 - newtondebugdisplaymode;
	if(newtondebugdisplaymode == 1) newton_debugdisplay();
}

//--------------------------------------------------------------------------------------- newton_debugdisplay_startup()
// register the key for toggling the debug display
//---------------------------------------------------------------------------------------
void newton_debugdisplay_startup()
{
	on_f12 = newton_toggledebugdisplay;
}